#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform layout(binding = 0) sampler2D diffuseTexture;
uniform layout(binding = 1) sampler2D lightTexture;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL1, pointL2;

uniform float brightness;
uniform bool hasLightmap;
uniform bool showLightmap;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	color = vec4(texColor * materialCoefficients.x, 1); // ambient
	//color = vec4(0.0f, 0.0f, 0.0f, 1.0f);
		
	if (hasLightmap && showLightmap) {
		vec4 s = texture(lightTexture, vert.uv);
		vec3 lightmapColor = vec3(s.x / 255.0f, s.y / 255.0f, s.z / 255.0f);
		color.rgb += lightmapColor;

	} else if (!hasLightmap) {
		// add point light contribution
		color.rgb += phong(n, pointL1.position - vert.position_world, v, pointL1.color * texColor, materialCoefficients.y, pointL1.color, materialCoefficients.z, specularAlpha, true, pointL1.attenuation);
		color.rgb += phong(n, pointL2.position - vert.position_world, v, pointL2.color * texColor, materialCoefficients.y, pointL2.color, materialCoefficients.z, specularAlpha, true, pointL2.attenuation);
	}
	
	color.rgb += brightness;
}

